import site
import os
import shutil
from pathlib import Path

import requests
import toml


def is_felucca_package(package: str):

    url = "https://pypi.org/pypi"
    response = requests.get(f"{url}/{package}/json").json()

    keywords = response["info"]["keywords"]
    return True if "felucca" in keywords else False


def find_dependency_version(package: str) -> str:

    lock = toml.load("./poetry.lock")
    for package_details in lock["package"]:
        if package in package_details["name"]:
            return package_details["version"]


def set_cairo_package(package: str, version: str):

    pyproject_file = "./pyproject.toml"
    settings = toml.load(pyproject_file)
    if "felucca" not in settings:
        settings["felucca"] = {}
        settings["felucca"]["contracts"] = {}
    settings["felucca"]["contracts"][f"{package}"] = version
    with open(pyproject_file, "w") as file:
        toml.dump(settings, file)


def remove_cairo_package(package):
    pyproject_file = "./pyproject.toml"
    settings = toml.load(pyproject_file)
    del settings["felucca"]["contracts"][f"{package}"]
    with open(pyproject_file, "w") as file:
        toml.dump(settings, file)


def get_package_name() -> str:
    pyproject_file = "./pyproject.toml"
    settings = toml.load(pyproject_file)
    return settings["tool"]["poetry"]["name"]


def install_cairo_package(package):
    site_packages = site.getsitepackages()[0]
    norm_package = package.replace("-", "_")
    package_contracts = os.path.join(site_packages, norm_package)
    target_dir = f"./{get_package_name()}/{norm_package}"
    shutil.copytree(package_contracts, target_dir)

    for contract in Path(target_dir).glob("**/*.cairo"):
        content = contract.read_text()
        content = content.replace(
            f"from {norm_package}.", f"from {get_package_name()}.{norm_package}."
        )
        contract.write_text(content)


def uninstall_cairo_package(package):

    norm_package = package.replace("-", "_")
    target_dir = f"./{get_package_name()}/{norm_package}"
    shutil.rmtree(target_dir)



from struct import pack
import subprocess

import typer
from rich.console import Console

from felucca.utils import (
    find_dependency_version,
    install_cairo_package,
    is_felucca_package,
    set_cairo_package,
    uninstall_cairo_package,
)

app = typer.Typer()
_console = Console()


@app.command()
def install(package: str):
    if is_felucca_package(package):
        with _console.status(f"[bold green] Installing {package}..."):
            result = subprocess.run(["poetry", "add", package, "--quiet"])
        if result.returncode == 0:
            _console.print(
                f"Done! [bold]{package}[/bold] successfully installed. Find installed Cairo contracts in the `contracts` folder :rocket:"
            )
            version = find_dependency_version(package)
            set_cairo_package(package, version)
            install_cairo_package(package)
    else:
        _console.print(
            f"Installation aborted. [bold]{package}[/bold] is not a Cairo package :sweat:"
        )


@app.command()
def uninstall(package: str):
    if is_felucca_package(package):
        with _console.status(f"[bold green] Uninstalling {package}..."):
            result = subprocess.run(["poetry", "remove", package, "--quiet"])
        if result.returncode == 0:
            _console.print(
                "Done! {package} successfully uninstalled. Cairo contracts removed from `contracts` folder :rocket:"
            )
            version = find_dependency_version(package)
            set_cairo_package(package, version)
            uninstall_cairo_package(package)


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['felucca']

package_data = \
{'': ['*']}

install_requires = \
['click==8.0.4',
 'requests>=2.27.1,<3.0.0',
 'rich>=12.0.1,<13.0.0',
 'toml>=0.10.2,<0.11.0',
 'typer>=0.4.1,<0.5.0']

entry_points = \
{'console_scripts': ['felucca = felucca.main:app']}

setup_kwargs = {
    'name': 'felucca',
    'version': '0.0.7',
    'description': 'Cairo dependency management and packaging made easy.',
    'long_description': '# Felucca: Dependency Management for Cairo\n\nFelucca helps you declare, manage and install dependencies of Cairo projects, ensuring you have the right stack everywhere.\n\n',
    'author': 'Fran Algaba',
    'author_email': 'f.algaba.work@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.11,<4.0.0',
}


setup(**setup_kwargs)
